require "ferrum"

def sigterm(pid)
  if Gem.win_platform?
    system("taskkill /F /pid #{pid}")
  else
    Process.kill("TERM", pid)
    Process.wait(pid)
  end
rescue Errno::ECHILD
  # Ignore
end

def free_port
  $free_port ||= 19900
  $free_port += 1
  $free_port
end

browser_options = {
  timeout: 15,
  process_timeout: 20,
  js_errors: true,
  browser_options: {
    'no-sandbox': nil,
    'auto-open-devtools-for-tabs': nil
  }
}

RSpec.describe "Example" do
  Dir[__dir__+"/../examples/*"].each do |example|
    example_name = File.basename(example)

    example_specific = proc do |browser, debug, error|
      case example_name
      when "opal-doc"
        browser.evaluate("typeof(Opal.Kernel.$puts)").should be == "function"
      when "opal-exception"
        error.should_not be_nil
        functions = error.stack_trace["callFrames"].map { |i| i["functionName"] }
        functions.should include "$$a"
        functions.should include "$$b"
        functions.should include "$$c"
        # How can we now test that source maps work? The browser doesn't help us...
      when "sass"
        browser.evaluate("window.getComputedStyle(document.body).backgroundColor").should be == "rgb(0, 128, 0)"
      end
    end

    context example_name do
      it "works in development mode" do
        Dir.chdir example do
          Bundler.with_unbundled_env do
            if ENV.key? 'CI'
              # Ensure we will reuse the cached gems
              system("bundle config --local path #{__dir__+"/../vendor/bundle"}")
            end

            system("bundle install").should be true

            begin
              port = free_port
              pid = spawn("bundle exec rackup -p#{port}")
              sleep 1

              begin
                browser = Ferrum::Browser.new(**browser_options)
                browser.go_to("http://localhost:#{port}/")
                browser.network.wait_for_idle

                # Sometimes we need to wait a little bit longer for the exception to appear.
                sleep 3 if example_name == "opal-exception"

                instance_exec(browser, true, nil, &example_specific)
              rescue Ferrum::JavaScriptError => e
                instance_exec(browser, true, e, &example_specific)
              ensure
                browser.quit
              end
            ensure
              sigterm(pid)
            end
          end
        end
      end

      it "works in production mode" do
        Dir.chdir example do
          Bundler.with_unbundled_env do
            ENV['RACK_ENV'] = 'production'

            if ENV.key? 'CI'
              system("bundle config --local path #{__dir__+"/../vendor/bundle"}")
            end

            system("bundle install").should be true

            system("bundle exec rake assets:clean").should be true
            -> { Dir.children(example + "/public/assets") }.should raise_error Errno::ENOENT
            system("bundle exec rake assets:precompile").should be true
            Dir.children(example + "/public/assets").length.should be == 3

            begin
              port = free_port
              pid = spawn("bundle exec rackup -p#{port}")
              sleep 1

              begin
                browser = Ferrum::Browser.new(**browser_options)
                browser.go_to("http://localhost:#{port}/")
                browser.network.wait_for_idle

                instance_exec(browser, false, nil, &example_specific)
              rescue Ferrum::JavaScriptError => e
                instance_exec(browser, true, e, &example_specific)
              ensure
                browser.quit
              end
            ensure
              system("bundle exec rake assets:clean").should be true
              -> { Dir.children(example + "/public/assets") }.should raise_error Errno::ENOENT

              sigterm(pid)
            end
          ensure
            ENV.delete('RACK_ENV')
          end
        end
      end
    end
  end
end