require "time"
require "roda/plugins/sprockets"

RSpec.shared_examples("basic checks") do
  it "returns a OK result" do
    req = app.req
    req.should be_ok
  end

  it "generates the necessary tags" do
    req = app.req
    if debug
      req.body.should include %'<script src="/assets/#{js_name}.debug.js" type="text/javascript"'
      req.body.should include %'<link rel="stylesheet" type="text/css" href="/assets/#{css_name}.debug.css">'
    else
      req.body.should include %'<script src="/assets/#{js_name}-'
      req.body.should include %'<link rel="stylesheet" type="text/css" href="/assets/#{css_name}-'
    end
  end

  it "generates the necessary files" do
    req = app.req
    req_css = app.req(req.css_path)
    req_js = app.req(req.js_path)

    req_css.should be_ok
    req_css.body.should include css_value

    req_js.should be_ok
    req_js.body.should include js_value
  end

  it "returns a not modified status if the assets haven't been modified" do
    req = app.req

    etag_css = app.req(req.css_path).headers["ETag"]
    etag_js = app.req(req.js_path).headers["ETag"]

    req_css = app.req(req.css_path, "HTTP_IF_NONE_MATCH" => etag_css)
    req_js = app.req(req.js_path, "HTTP_IF_NONE_MATCH" => etag_js)

    req_css.should be_not_modified
    req_js.should be_not_modified
  end

  it "appropriately includes source maps or not" do
    req = app.req
    req_css = app.req(req.css_path)
    req_js = app.req(req.js_path)

    if debug
      req_css.body.should include "/*# sourceMappingURL="
      req_js.body.should include "//# sourceMappingURL="
    else
      req_css.body.should_not include "/*# sourceMappingURL="
      req_js.body.should_not include "//# sourceMappingURL="
    end
  end
end

RSpec.describe Roda::RodaPlugins::Sprockets do
  context "simple app" do
    let(:css_name) { "simple_css" }
    let(:js_name) { "simple_js" }
    let(:css_value) { "Simple CSS" }
    let(:js_value) { "Simple JS" }

    context "in debug mode" do
      simple_debug_app = Server.gen("simple_css", "simple_js", debug: true)

      let(:app) { simple_debug_app }
      let(:debug) { true }

      include_examples "basic checks"
    end

    context "in production mode" do
      simple_prod_app = Server.gen("simple_css", "simple_js", debug: false)

      let(:app) { simple_prod_app }
      let(:debug) { false }

      include_examples "basic checks"
    end
  end

  context "opal + scss app" do
    let(:css_name) { "scss_css" }
    let(:js_name) { "opal_js" }
    let(:css_value) { "This is SCSS" }
    let(:js_value) { "$$($nesting, 'Test').$new();" }

    context "in debug mode" do
      opalscss_debug_app = Server.gen("scss_css", "opal_js", opal: true, debug: true)

      let(:app) { opalscss_debug_app }
      let(:debug) { true }

      include_examples "basic checks"
    end

    context "in production mode" do
      opalscss_prod_app = Server.gen("scss_css", "opal_js", opal: true, debug: false)

      let(:app) { opalscss_prod_app }
      let(:debug) { false }

      include_examples "basic checks"
    end
  end

  context "opal with requires + scss with imports app" do
    let(:css_name) { "scss_css_with_imports" }
    let(:js_name) { "opal_js_with_requires" }
    let(:css_value) { "This is SCSS" }
    let(:js_value) { "$$($nesting, 'Test').$new();" }

    context "in debug mode" do
      opalscssreq_debug_app = Server.gen("scss_css_with_imports", "opal_js_with_requires", opal: true, debug: true)

      let(:app) { opalscssreq_debug_app }
      let(:debug) { true }

      include_examples "basic checks"
    end

    context "in production mode" do
      opalscssreq_prod_app = Server.gen("scss_css_with_imports", "opal_js_with_requires", opal: true, debug: false)

      let(:app) { opalscssreq_prod_app }
      let(:debug) { false }

      include_examples "basic checks"
    end
  end

  context "opal with erb + scss with erb app" do
    let(:css_name) { "scss_css_with_erb" }
    let(:js_name) { "opal_js_with_erb" }
    let(:css_value) { "This is ERBSCSS" }
    let(:js_value) { "$$($nesting, 'TestERB').$new();" }

    context "in debug mode" do
      opalscsserb_debug_app = Server.gen("scss_css_with_erb", "opal_js_with_erb", opal: true, debug: true)

      let(:app) { opalscsserb_debug_app }
      let(:debug) { true }

      include_examples "basic checks"
    end

    context "in production mode" do
      opalscsserb_prod_app = Server.gen("scss_css_with_erb", "opal_js_with_erb", opal: true, debug: false)

      let(:app) { opalscsserb_prod_app }
      let(:debug) { false }

      include_examples "basic checks"
    end
  end

  context "opal + scss app with compressor" do
    let(:css_name) { "scss_css" }
    let(:js_name) { "opal_js" }
    let(:css_value) { "This is SCSS" }
    let(:js_value) { "Test" }

    context "in debug mode" do
      opalscsscomp_debug_app = Server.gen("scss_css", "opal_js",
        css_compressor: :sassc, js_compressor: :uglify,
        opal: true, debug: true
      )

      let(:app) { opalscsscomp_debug_app }
      let(:debug) { true }

      include_examples "basic checks"

      it "should run uglifier" do
        req = app.req
        req_js = app.req(req.js_path)

        req_js.body.should include "$$($nesting, 'Test').$new();"
      end
    end

    context "in production mode" do
      opalscsscomp_prod_app = Server.gen("scss_css", "opal_js",
        css_compressor: :sassc, js_compressor: :uglify,
        opal: true, debug: false
      )

      let(:app) { opalscsscomp_prod_app }
      let(:debug) { false }

      include_examples "basic checks"

      it "should not run uglifier" do
        req = app.req
        req_js = app.req(req.js_path)

        req_js.body.should_not include "$$($nesting, 'Test').$new();"
      end
    end
  end

  context "opal + scss app with cache" do
    let(:css_name) { "scss_css" }
    let(:js_name) { "opal_js" }
    let(:css_value) { "This is SCSS" }
    let(:js_value) { "$$($nesting, 'Test').$new();" }

    context "in debug mode" do
      opalscsscache_debug_app = Server.gen("scss_css", "opal_js",
        opal: true, debug: true,
        cache: Sprockets::Cache::MemoryStore.new(65536)
      )

      let(:app) { opalscsscache_debug_app }
      let(:debug) { true }

      include_examples "basic checks"
    end

    context "in production mode" do
      opalscsscache_prod_app = Server.gen("scss_css", "opal_js",
        opal: true, debug: false,
        cache: Sprockets::Cache::MemoryStore.new(65536)
      )

      let(:app) { opalscsscache_prod_app }
      let(:debug) { false }

      include_examples "basic checks"
    end
  end
end